@@ -19,6 +19,9 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
+import os
+import collections
+
 from . import keywords
 from . import tokenize
 
@@ -94,11 +97,18 @@ class Define(Node):
         Node.__init__(self, start, end)
         self.name = name
         self.definition = definition
+        # TODO(christarazi):
+        # Defines aren't bound to namespaces, so this is just a stopgap
+        # solution.
+        self.namespace = []
 
     def __str__(self):
         value = '%s %s' % (self.name, self.definition)
         return self._string_helper(self.__class__.__name__, value)
 
+    def is_exportable(self):
+        return True
+
 
 class Include(Node):
 
@@ -271,7 +281,10 @@ class Union(Class):
 class Function(_GenericDeclaration):
 
     def __init__(self, start, end, name, return_type, parameters,
-                 specializations, modifiers, templated_types, body, namespace):
+                 specializations, modifiers, templated_types, body, namespace,
+                 initializers=None):
+        if initializers is None:
+            initializers = {}
         _GenericDeclaration.__init__(self, start, end, name, namespace)
         converter = TypeConverter(namespace)
         self.return_type = converter.create_return_type(return_type)
@@ -280,6 +293,7 @@ class Function(_GenericDeclaration):
         self.modifiers = modifiers
         self.body = body
         self.templated_types = templated_types
+        self.initializers = initializers
 
     def is_declaration(self):
         return self.body is None
@@ -308,6 +322,14 @@ class Method(Function):
 
     def __init__(self, start, end, name, in_class, return_type, parameters,
                  specializations, modifiers, templated_types, body, namespace):
+        # TODO(christarazi):
+        # Add support for ctor initializers.
+        # For now, only inline defined ctors are supported because we would
+        # need to figure out how to keep state of which class is currently
+        # being processed in order to modify its body (var decls).
+        #
+        # Note: ctor initializers are inside Function because a inline defined
+        # class members are parsed as functions rather than methods.
         Function.__init__(self, start, end, name, return_type, parameters,
                           specializations, modifiers, templated_types,
                           body, namespace)
@@ -328,7 +350,6 @@ class Type(_GenericDeclaration):
         templated_types: [Class (Type?)] template type info between <>
         modifiers: [str] type modifiers (keywords) eg, const, mutable, etc.
         reference, pointer, array: bools
-
         """
         _GenericDeclaration.__init__(self, start, end, name, [])
         self.templated_types = templated_types
@@ -392,7 +413,6 @@ class TypeConverter(object):
 
         Returns:
           [Class(...), ...]
-
         """
         result = []
         name_tokens = []
@@ -613,11 +633,14 @@ class TypeConverter(object):
 
 class ASTBuilder(object):
 
-    def __init__(self, token_stream, filename, in_class=None,
+    def __init__(self, token_stream, filename, system_includes=tuple(),
+                 nonsystem_includes=tuple(), in_class=None,
                  namespace_stack=None, quiet=False):
         if namespace_stack is None:
             namespace_stack = []
 
+        self.system_includes = system_includes
+        self.nonsystem_includes = nonsystem_includes
         self.tokens = token_stream
         self.filename = filename
         self.token_queue = []
@@ -644,13 +667,17 @@ class ASTBuilder(object):
                 self.namespaces.append(False)
                 continue
             if token.name == '}':
-                if self.namespaces.pop():
+                if self.namespaces and self.namespaces.pop():
                     self.namespace_stack.pop()
                 continue
 
-            result = self._generate_one(token)
-            if result:
-                yield result
+            try:
+                result = self._generate_one(token)
+            except StopIteration:
+                pass
+            else:
+                if result:
+                    yield result
 
     def _create_variable(self, pos_token, name, type_name, type_modifiers,
                          ref_pointer_name_seq, templated_types=None, value=''):
@@ -742,13 +769,24 @@ class ASTBuilder(object):
                 if name.startswith('\\'):
                     name = name[1:].strip()
 
-                system = True
-                filename = name
-                if name[0] in '<"':
-                    assert_parse(name[-1] in '>"', token)
+                filename = name.strip('<>"')
+
+                def _is_file(prefix):
+                    return os.path.isfile(os.path.join(prefix, filename))
+
+                if any([d for d in self.system_includes if _is_file(d)]):
+                    system = True
+                elif any([d for d in self.nonsystem_includes if _is_file(d)]):
+                    system = False
+                else:
+                    system = True
+                    filename = name
+
+                    if name[0] in '<"':
+                        assert_parse(name[-1] in '>"', token)
+                        system = name[0] == '<'
+                        filename = name[1:-1]
 
-                    system = name[0] == '<'
-                    filename = name[1:-1]
                 return Include(token.start, token.end, filename, system)
             if name.startswith('define'):
                 # Remove "define".
@@ -786,6 +824,26 @@ class ASTBuilder(object):
         return self._get_var_tokens_up_to(False,
                                           expected_token)[0]
 
+    def _get_var_tokens_up_to_w_function(self, skip_bracket_content,
+                                         *expected_tokens):
+        # handle the std::function and boost::function case
+        tokens, last = self._get_var_tokens_up_to(False, *expected_tokens)
+        names = [token.name for token in tokens]
+        ctr = collections.Counter(names)
+        if ('(' in expected_tokens and
+            ctr['<'] != ctr['>'] and
+            ctr['function'] == 1 and
+                last.name == '('):
+
+            idx = names.index('function')
+            if idx + 1 < len(tokens) and tokens[idx + 1].name == '<':
+                new_tokens, new_last = \
+                    self._get_var_tokens_up_to(False, '(', ';')
+                tokens.append(last)
+                last = new_last
+                tokens += new_tokens
+        return tokens, last
+
     def _get_var_tokens_up_to(self, skip_bracket_content, *expected_tokens):
         last_token = self._get_next_token()
         tokens = []
@@ -883,7 +941,8 @@ class ASTBuilder(object):
         return tokens, next_token
 
     def get_method(self, modifiers, templated_types):
-        return_type_and_name = self._get_tokens_up_to('(')
+        return_type_and_name = \
+            self._get_var_tokens_up_to_w_function(False, '(')[0]
         assert len(return_type_and_name) >= 1
         return self._get_method(
             return_type_and_name, modifiers, templated_types,
@@ -972,11 +1031,17 @@ class ASTBuilder(object):
             if (
                 token.name == 'const' or
                 token.name == 'override' or
-                token.name == 'final' or
-                token.name == 'noexcept'
+                token.name == 'final'
             ):
                 modifiers |= FUNCTION_SPECIFIER
                 token = self._get_next_token()
+            elif token.name == 'noexcept':
+                modifiers |= FUNCTION_SPECIFIER
+                token = self._get_next_token()
+                if token.name == '(':
+                    # Consume everything between the parens.
+                    list(self._get_matching_char('(', ')'))
+                    token = self._get_next_token()
             elif token.name == '__attribute__':
                 # TODO(nnorwitz): handle more __attribute__ details.
                 modifiers |= FUNCTION_ATTRIBUTE
@@ -1021,39 +1086,50 @@ class ASTBuilder(object):
         assert_parse(token.token_type == tokenize.SYNTAX, token)
 
         # Handle ctor initializers.
+        # Supports C++11 method of direct initialization with braces.
+        initializers = {}
         if token.name == ':':
             while token.name != ';' and token.name != '{':
-                _, token = self.get_name()
-                if token.name == '(':
-                    list(self._get_matching_char('(', ')'))
-                elif token.name == '{':
-                    list(self._get_matching_char('{', '}'))
+                # skip preprocesors macros
+                if token.name.startswith(('#if', '#elif', '#else', '#endif')):
+                    token = self._get_next_token()
+                    continue
+                member, token = self.get_name()
+                member = member[0]
+                if token.name == '(' or token.name == '{':
+                    end = '}' if token.name == '{' else ')'
+                    initializers[member] = [
+                        x for x in list(self._get_matching_char(token.name,
+                                                                end))
+                        if x.name != ',' and x.name != end]
                 token = self._get_next_token()
 
-        # Handle pointer to functions that are really data but look
-        # like method declarations.
+        # Handle pointer to functions.
         if token.name == '(':
             # name contains the return type.
             return_type.append(name)
-            name = parameters.pop()
+            while parameters[-1].name in '()':
+                parameters.pop()
+            name = parameters[-1]
             # Already at the ( to open the parameter list.
-            function_parameters = list(self._get_matching_char('(', ')'))
-            del function_parameters[-1]  # Remove trailing ')'.
+            parameters = list(self._get_matching_char('(', ')'))
+            del parameters[-1]  # Remove trailing ')'.
             # TODO(nnorwitz): store the function_parameters.
             token = self._get_next_token()
 
-            default = []
-            if token.name == '=':
-                default.extend(self._get_tokens_up_to(';'))
+            if token.name != '{':
+                default = []
+                if token.name == '=':
+                    default.extend(self._get_tokens_up_to(';'))
 
-            return self._create_variable(
-                indices,
-                name.name,
-                indices.name,
-                [],
-                [t.name for t in return_type],
-                None,
-                ''.join([t.name for t in default]))
+                return self._create_variable(
+                    indices,
+                    name.name,
+                    indices.name,
+                    [],
+                    [t.name for t in return_type],
+                    None,
+                    ''.join([t.name for t in default]))
 
         if token.name == '{':
             body = list(self.get_scope())
@@ -1092,7 +1168,8 @@ class ASTBuilder(object):
                           templated_types, body, self.namespace_stack)
         return Function(indices.start, indices.end, name.name, return_type,
                         parameters, specializations, modifiers,
-                        templated_types, body, self.namespace_stack)
+                        templated_types, body, self.namespace_stack,
+                        initializers)
 
     def _get_variable(self, tokens):
         name, type_name, templated_types, modifiers, default, _ = \
@@ -1354,7 +1431,8 @@ class ASTBuilder(object):
         templated_types = None
         if token.token_type == tokenize.SYNTAX and token.name == '<':
             templated_types = self._get_templated_types()
-            # TODO(nnorwitz): for now, just ignore the template params.
+            token = self._get_next_token()
+        while token.token_type == tokenize.PREPROCESSOR:
             token = self._get_next_token()
 
         if token.token_type == tokenize.NAME:
@@ -1364,12 +1442,15 @@ class ASTBuilder(object):
             elif token.name == 'struct':
                 return self._get_class(Struct,
                                        templated_types)
+            elif token.name == 'union':
+                return self._get_class(Union,
+                                       templated_types)
             elif token.name == 'friend':
                 return self.handle_friend()
             elif token.name == 'template':
                 return self.handle_template()
         self._add_back_token(token)
-        tokens, last = self._get_var_tokens_up_to(False, '(', ';')
+        tokens, last = self._get_var_tokens_up_to_w_function(False, '(', ';')
         tokens.append(last)
         self._add_back_tokens(tokens)
         if last.name == '(':
@@ -1380,29 +1461,34 @@ class ASTBuilder(object):
     def _get_bases(self):
         # Get base classes.
         bases = []
+        specifier = ('public', 'protected', 'private', 'virtual')
         while True:
             token = self._get_next_token()
-            assert_parse(token.token_type == tokenize.NAME, token)
-            if token.name == 'virtual':
-                token = self._get_next_token()
-            # TODO(nnorwitz): store kind of inheritance...maybe.
-            if token.name not in ('public', 'protected', 'private'):
-                # If inheritance type is not specified, it is private.
-                # Just put the token back so we can form a name.
-                # TODO(nnorwitz): it would be good to warn about this.
-                self._add_back_token(token)
-            else:
-                # Check for virtual inheritance.
-                token = self._get_next_token()
-                if token.name != 'virtual':
-                    self._add_back_token(token)
+            if (
+                token.name in specifier or
+                token.token_type == tokenize.PREPROCESSOR
+            ):
+                continue
+            self._add_back_token(token)
+
             base, next_token = self.get_name()
+            if (
+                len(base) > 2 and
+                base[-2].name == '::' and
+                next_token.token_type == tokenize.NAME and
+                next_token.name not in specifier
+            ):
+                self._add_back_token(next_token)
+                base2, next_token = self.get_name()
+                base.pop()
+                base.extend(base2)
             bases_ast = self.converter.to_type(base)
-            assert_parse(len(bases_ast) == 1, bases_ast)
-            bases.append(bases_ast[0])
-            if next_token.token_type == tokenize.PREPROCESSOR:
+            if len(bases_ast) == 1:
+                bases.append(bases_ast[0])
+            if next_token.name == ')':
+                next_token = self._get_next_token()
+            while next_token.token_type == tokenize.PREPROCESSOR:
                 next_token = self._get_next_token()
-            assert_parse(next_token.token_type == tokenize.SYNTAX, next_token)
             if next_token.name == '{':
                 token = next_token
                 break
@@ -1411,6 +1497,7 @@ class ASTBuilder(object):
     def _get_class(self, class_type, templated_types):
         class_name = None
         class_token = self._get_next_token()
+        name_tokens = []
         if class_token.token_type != tokenize.NAME:
             assert_parse(class_token.token_type == tokenize.SYNTAX,
                          class_token)
@@ -1463,17 +1550,42 @@ class ASTBuilder(object):
         body = None
         if token.token_type == tokenize.SYNTAX and token.name == '{':
             name = class_name or '__unamed__'
-            ast = ASTBuilder(self.get_scope(), self.filename, name,
+            ast = ASTBuilder(self.get_scope(), self.filename,
+                             self.system_includes, self.nonsystem_includes,
+                             name,
                              self.namespace_stack,
                              quiet=self.quiet)
             body = list(ast.generate())
 
+            ctor = None
+            for member in body:
+                if isinstance(member, Function) and member.name == class_name:
+                    ctor = member
+                    break
+
+            # Merge ctor initializers with class members.
+            if ctor:
+                initializers = body[body.index(ctor)].initializers
+                var_decls = [x for x in body
+                             if isinstance(x, VariableDeclaration)]
+                for var in var_decls:
+                    for key, val in initializers.items():
+                        # TODO: CT
+                        # In the future, support members that have ctors with
+                        # more than one parameter.
+                        if len(val) > 1:
+                            continue
+                        if len(val) == 1 and var.name == key.name:
+                            body[body.index(var)].initial_value = val[0].name
+
             if not self._handling_typedef:
                 token = self._get_next_token()
                 if token.token_type != tokenize.NAME:
                     assert_parse(token.token_type == tokenize.SYNTAX, token)
                     assert_parse(token.name == ';', token)
                 else:
+                    if keywords.is_builtin_type(token.name):
+                        token = self._get_next_token()
                     self._ignore_up_to(';')
                     new_class = class_type(class_token.start, class_token.end,
                                            class_name, bases, None,
@@ -1514,7 +1626,18 @@ class ASTBuilder(object):
     def handle_using(self):
         tokens = self._get_tokens_up_to(';')
         assert tokens
-        return Using(tokens[0].start, tokens[0].end, tokens)
+        new_type = self.converter.to_type(tokens)
+        if 'namespace' in new_type[0].modifiers:
+            return Using(tokens[0].start, tokens[0].end, tokens)
+        else:
+            # aside from namespaces, "using" can be used just like a typedef
+            # e.g., the following lines are equivalent
+            # using Foo = Bar;
+            # typedef Bar Foo;
+            # There is already code written to handle the typedef case so
+            # we return a Typdef object
+            return Typedef(tokens[0].start, tokens[0].end, new_type[0].name,
+                           new_type, self.namespace_stack)
 
     def handle_explicit(self):
         assert self.in_class
@@ -1528,7 +1651,8 @@ class ASTBuilder(object):
         pass
 
 
-def builder_from_source(source, filename, quiet=False):
+def builder_from_source(source, filename, system_includes,
+                        nonsystem_includes, quiet=False):
     """Utility method that returns an ASTBuilder from source code.
 
     Args:
@@ -1537,10 +1661,11 @@ def builder_from_source(source, filename, quiet=False):
 
     Returns:
       ASTBuilder
-
     """
     return ASTBuilder(tokenize.get_tokens(source),
                       filename,
+                      system_includes,
+                      nonsystem_includes,
                       quiet=quiet)
 
 
