@@ -20,7 +20,6 @@ be generated and which should be suppressed. Currently, all possible
 warnings will always be displayed. There is no way to suppress any.
 There also needs to be a way to use annotations in the source code to
 suppress warnings.
-
 """
 
 from __future__ import absolute_import
@@ -51,7 +50,7 @@ __author__ = 'nnorwitz@google.com (Neal Norwitz)'
 HEADER_EXTENSIONS = frozenset(['.h', '.hh', '.hpp', '.h++', '.hxx', '.cuh'])
 CPP_EXTENSIONS = frozenset(['.cc', '.cpp', '.c++', '.cxx', '.cu'])
 
-# These enumerations are used to determine how an symbol/#include file is used.
+# These enumerations are used to determine how a symbol/#include file is used.
 UNUSED = 0
 USES_REFERENCE = 1
 USES_DECLARATION = 2
@@ -61,7 +60,7 @@ DECLARATION_TYPES = (ast.Class, ast.Struct, ast.Enum, ast.Union)
 
 class Module(object):
 
-    """Data container represting a single source file."""
+    """Data container representing a single source file."""
 
     def __init__(self, filename, ast_list):
         self.filename = filename
@@ -89,11 +88,15 @@ class WarningHunter(object):
     # Cache filename: ast_list
     _module_cache = {}
 
-    def __init__(self, filename, source, ast_list, include_paths, quiet=False):
+    def __init__(self, filename, source, ast_list, include_paths,
+                 system_include_paths, nonsystem_include_paths,
+                 quiet=False):
         self.filename = filename
         self.source = source
         self.ast_list = ast_list
         self.include_paths = include_paths[:]
+        self.system_include_paths = system_include_paths
+        self.nonsystem_include_paths = nonsystem_include_paths
         self.quiet = quiet
         self.symbol_table = symbols.SymbolTable()
 
@@ -147,6 +150,8 @@ class WarningHunter(object):
             ast_list = None
             try:
                 builder = ast.builder_from_source(source, filename,
+                                                  self.system_include_paths,
+                                                  self.nonsystem_include_paths,
                                                   quiet=self.quiet)
                 ast_list = [_f for _f in builder.generate() if _f]
             except tokenize.TokenError:
@@ -243,11 +248,23 @@ class WarningHunter(object):
                 decl_uses[name] |= USES_REFERENCE
             except symbols.Error:
                 module = Module(name, None)
-                self.symbol_table.add_symbol(node.name, node.namespace, node,
-                                             module)
+                symbol_table.add_symbol(node.name, node.namespace, node,
+                                        module)
+
+        def _do_lookup(name, namespace):
+            try:
+                file_use_node = symbol_table.lookup_symbol(name, namespace)
+            except symbols.Error:
+                return
+            name = file_use_node[1].filename
+            file_uses[name] = file_uses.get(name, 0) | USES_DECLARATION
 
         def _add_declaration(name, namespace):
-            assert name
+            if not name:
+                # Ignore anonymous struct. It is not standard, but we might as
+                # well avoid crashing if it is easy.
+                return
+
             names = [n for n in namespace if n is not None]
             if names:
                 name = '::'.join(names) + '::' + name
@@ -265,38 +282,55 @@ class WarningHunter(object):
                 decl_uses[name] |= USES_REFERENCE
             elif name in file_uses:
                 # enum and typedef can't be forward declared
-                if (
-                    isinstance(file_use_node[0], ast.Enum) or
-                    isinstance(file_use_node[0], ast.Typedef)
-                ):
+                if isinstance(file_use_node[0], (ast.Enum, ast.Typedef)):
                     file_uses[name] |= USES_DECLARATION
                 else:
                     file_uses[name] |= USES_REFERENCE
 
-        def _add_use(name, namespace):
-            if isinstance(name, list):
+        def _add_use(node, namespace, name=''):
+            if isinstance(node, basestring):
+                name = node
+            elif isinstance(node, list):
                 # name contains a list of tokens.
                 name = '::'.join([n.name for n in name])
-            elif not isinstance(name, basestring):
+
+            # node is a Type so look for its symbol immediately.
+            if name:
+                _do_lookup(name, namespace)
+                return
+
+            # Try to search for the value of the variable declaration for any
+            # symbols, such as `#define` values or other variable names which
+            # may be included in other files.
+            obj = getattr(node, 'initial_value', None)
+            if obj:
+                _do_lookup(obj, namespace)
+
+            # If node is a VariableDeclaration, check if the variable type is
+            # a symbol used in other includes.
+            obj = getattr(node, 'type', None)
+            if obj and isinstance(obj.name, basestring):
+                _do_lookup(obj.name, namespace)
+
+            if not isinstance(node, basestring):
                 # Happens when variables are defined with inlined types, e.g.:
                 #   enum {...} variable;
                 return
-            try:
-                file_use_node = symbol_table.lookup_symbol(name, namespace)
-            except symbols.Error:
-                return
-
-            name = file_use_node[1].filename
-            file_uses[name] = file_uses.get(name, 0) | USES_DECLARATION
 
         def _add_variable(node, namespace, reference=False):
-            if node.reference or node.pointer or reference:
-                _add_reference(node.name, namespace)
+            obj = node.type if isinstance(
+                node, ast.VariableDeclaration) else node
+
+            if obj.reference or obj.pointer or reference:
+                _add_reference(obj.name, namespace)
             else:
-                _add_use(node.name, namespace)
+                # Add a use for the variable declaration type as well as the
+                # variable value.
+                _add_use(obj.name, namespace)
+                _add_use(node, namespace)
             # This needs to recurse when the node is a templated type.
-            _add_template_use(node.name,
-                              node.templated_types,
+            _add_template_use(obj.name,
+                              obj.templated_types,
                               namespace,
                               reference)
 
@@ -314,7 +348,7 @@ class WarningHunter(object):
                 node = p.type
                 if node.name not in templated_types:
                     if function.body and p.name:
-                        # Assume that if the the function has a body and a name
+                        # Assume that if the function has a body and a name
                         # the parameter type is really used.
                         # NOTE(nnorwitz): this is over-aggressive. It would be
                         # better to iterate through the body and determine
@@ -363,6 +397,9 @@ class WarningHunter(object):
                     # These are things like auto_ptr which do
                     # not require the class definition, only decl.
                     _add_reference(cls.name, namespace)
+                elif name.startswith('Q') and name.endswith('Pointer'):
+                    # Special case templated classes from the Qt framework.
+                    _add_reference(cls.name, namespace)
                 else:
                     _add_use(cls.name, namespace)
                 _add_template_use(cls.name, cls.templated_types,
@@ -380,7 +417,7 @@ class WarningHunter(object):
             for node in ast_seq.pop():
                 if isinstance(node, ast.VariableDeclaration):
                     namespace = namespace_stack + node.namespace
-                    _add_variable(node.type, namespace)
+                    _add_variable(node, namespace)
                 elif isinstance(node, ast.Function):
                     namespace = namespace_stack + node.namespace
                     _process_function(node, namespace)
@@ -586,9 +623,12 @@ def get_correct_include_filename(filename, candidate_filenames):
     return None
 
 
-def run(filename, source, entire_ast, include_paths, quiet):
+def run(filename, source, entire_ast, include_paths,
+        system_include_paths, nonsystem_include_paths, quiet):
     hunter = WarningHunter(filename, source, entire_ast,
                            include_paths=include_paths,
+                           system_include_paths=system_include_paths,
+                           nonsystem_include_paths=nonsystem_include_paths,
                            quiet=quiet)
     hunter.find_warnings()
     hunter.show_warnings()
