@@ -60,7 +60,9 @@ def _find_warnings(filename, lines, ast_list, static_is_optional):
                 'constexpr' not in node.type.modifiers
             )
 
-            if is_not_const and (static_is_optional or is_static):
+            is_not_none_type = node.name is not None
+
+            if is_not_const and (static_is_optional or is_static) and is_not_none_type:
                 print_warning(node)
                 count += 1
         elif isinstance(node, ast.Function) and node.body:
@@ -110,7 +112,8 @@ def _find_unused_static_warnings(filename, lines, ast_list):
     return count
 
 
-def run(filename, source, entire_ast, include_paths, quiet):
+def run(filename, source, entire_ast, include_paths,
+        system_include_paths, nonsystem_include_paths, quiet):
     lines = metrics.Metrics(source)
 
     return (
